/*	$Header: /usr/people/sam/fax/recvfax/RCS/alter.c,v 1.7 93/03/08 15:29:12 sam Rel $
/*
 * Copyright (c) 1990, 1991, 1992, 1993 Sam Leffler
 * Copyright (c) 1991, 1992, 1993 Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * Sam Leffler and Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Sam Leffler and Silicon Graphics.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL SAM LEFFLER OR SILICON GRAPHICS BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */
#include "defs.h"

#include <sys/types.h>
#include <sys/file.h>
#include <stdarg.h>

#include "flock.h"

/*
 * Setup a job's description file for alteration.
 */
static FILE*
setupAlteration(Job* job, char* jobname)
{
    FILE* fp = fopen(job->qfile, "a+");
    if (fp == NULL) {
	syslog(LOG_ERR, "alter: cannot open %s (%m)", job->qfile);
	sendClient("openFailed", "%s", jobname);
	return (fp);
    }
    if (flock(fileno(fp), LOCK_EX|LOCK_NB) < 0) {
	syslog(LOG_INFO, "%s locked during alteration (%m)", job->qfile);
	sendClient("jobLocked", "%s", jobname);
	fclose(fp);
	return (NULL);
    }
    return (fp);
}

static void
reallyAlterJobTTS(Job* job, char* jobname, char* tts)
{
    if (tts) {
	FILE* fp = setupAlteration(job, jobname);
	if (fp) {
	    fprintf(fp, "tts:%s\n", tts);
	    (void) flock(fileno(fp), LOCK_UN);
	    (void) fclose(fp);
	    /* XXX at job for delayed submissions! */
	    notifyServer(job->modem, "JT%s %s", job->qfile, tts);
	    syslog(LOG_INFO, "ALTER %s TTS %s completed", job->qfile, tts);
	    sendClient("altered", "%s", jobname);
	}
    } else
	protocolBotch("no time-to-send specification.");
}

static void
reallyAlterJobNotification(Job* job, char* jobname, char* note)
{
    if (note) {
	FILE* fp = setupAlteration(job, jobname);
	if (fp) {
	    fprintf(fp, "notify:%s\n", note);
	    (void) flock(fileno(fp), LOCK_UN);
	    (void) fclose(fp);
	    /* XXX: no server notification */
	    syslog(LOG_INFO, "ALTER %s NOTIFY %s completed", job->qfile, note);
	    sendClient("altered", "%s", jobname);
	}
    } else
	protocolBotch("no notification specification.");
}

void
alterJobTTS(const char* modem, char* tag)
{
    applyToJob(modem, tag, "alter", reallyAlterJobTTS);
}

void
alterJobNotification(const char* modem, char* tag)
{
    applyToJob(modem, tag, "alter", reallyAlterJobNotification);
}
