/*	$Header: /usr/people/sam/fax/recvfax/RCS/jobs.c,v 1.17 93/03/08 15:29:21 sam Rel $
/*
 * Copyright (c) 1990, 1991, 1992, 1993 Sam Leffler
 * Copyright (c) 1991, 1992, 1993 Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * Sam Leffler and Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Sam Leffler and Silicon Graphics.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL SAM LEFFLER OR SILICON GRAPHICS BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */
#include "defs.h"

#include <sys/types.h>
#include <sys/file.h>
#include <fcntl.h>
#include <dirent.h>
#include <malloc.h>
#include <pwd.h>
#include <errno.h>

#include "flock.h"

static Job*
readJob(Job* job)
{
    char line[1024];
    char* cp;
    char* tag;
    FILE* fp;

    job->flags = 0;
    fp = fopen((char*) job->qfile, "r");
    if (fp == NULL)
	return NULL;
    if (flock(fileno(fp), LOCK_SH|LOCK_NB) < 0 && errno == EWOULDBLOCK)
	job->flags |= JOB_LOCKED;
    job->status = NULL;
    job->modem = NULL;
    job->tts = 0;
    while (fgets(line, sizeof (line) - 1, fp)) {
	if (line[0] == '#')
	    continue;
	if (cp = strchr(line, '\n'))
	    *cp = '\0';
	tag = strchr(line, ':');
	if (!tag || !*tag)
	    continue;
	*tag++ = '\0';
	while (isspace(*tag))
	    tag++;
	if (isCmd("tts")) {
	    job->tts = atoi(tag);
	} else if (isCmd("killtime")) {
	    job->killtime = strdup(tag);
	} else if (isCmd("number")) {
	    job->number = strdup(tag);
	} else if (isCmd("sender")) {
	    job->sender = strdup(tag);
	} else if (isCmd("mailaddr")) {
	    job->mailaddr = strdup(tag);
	} else if (isCmd("status")) {
	    job->status = strdup(tag);
	} else if (isCmd("modem")) {
	    job->modem = strdup(tag);
	} else if (isCmd("sendAt")) {
	    job->sendat = strdup(tag);
	    job->flags |= JOB_SENDAT;
	}
    }
    if (job->status == NULL)
	job->status = strdup("");
    if (job->modem == NULL)
	job->modem = strdup(MODEM_ANY);
    if ((job->flags & JOB_LOCKED) == 0)
	(void) flock(fileno(fp), LOCK_UN);
    fclose(fp);
    return job;
}

Job*
readJobs()
{
    DIR* dirp;
    struct dirent* dentp;
    Job* jobs = 0;
    Job* jobp = 0;

    if (!(dirp = opendir(FAX_SENDDIR))) {
	syslog(LOG_ERR, "%s: opendir: %m", FAX_SENDDIR);
	sendError("Problem accessing send directory.");
	done(-1, "EXIT");
    }
    (void) flock(dirp->dd_fd, LOCK_SH);
    for (dentp = readdir(dirp); dentp; dentp = readdir(dirp)) {
	if (dentp->d_name[0] != 'q')
	    continue;
	jobp = (Job*) malloc(sizeof(Job));
	jobp->qfile = malloc(strlen(FAX_SENDDIR) + strlen(dentp->d_name) + 2);
	sprintf(jobp->qfile, "%s/%s", FAX_SENDDIR, dentp->d_name);
	if (readJob(jobp)) {
	    jobp->next = jobs;
	    jobs = jobp;
	} else {
	    free(jobp);
	}
    }
    (void) flock(dirp->dd_fd, LOCK_UN);
    closedir(dirp);
    return jobs;
}

int
modemMatch(const char* a, const char* b)
{
    return strcmp(a, MODEM_ANY) == 0 || strcmp(b, MODEM_ANY) == 0 ||
	strcmp(a, b) == 0;
}

void
applyToJob(const char* modem, char* tag, char* op, jobFunc* f)
{
    Job** job;
    struct passwd* pwd = NULL;
    char buf[1024];
    char* cp;
    char* requestor;
    char* arg;

    if (!jobList)
	jobList = readJobs();

    if ((requestor = strchr(tag, ':')) == 0) {
	protocolBotch("no requestor name for \"%s\" command.", op);
	done(1, "EXIT");
    }
    *requestor++ = '\0';
    arg = strchr(requestor, ':');
    if (arg)
	*arg++ = '\0';

    for (job = &jobList; *job; job = &((*job)->next)) {
	char *jobname = (*job)->qfile+strlen(FAX_SENDDIR)+2;
	if (modemMatch(modem, (*job)->modem) && strcmp(jobname, tag) == 0)
	    break;
    }
    if (!*job) {
	sendClient("notQueued", "%s", tag);
	return;			/* nothing to do */
    }
    if (strcmp(requestor, (*job)->sender) == 0) {
	/* alter job parameters */
	if (debug)
	    syslog(LOG_DEBUG, "%s request by owner for %s", op, tag);
	(*f)(*job, tag, arg);
	if ((*job)->flags & JOB_INVALID)
	    *job = (*job)->next;
	return;			/* nothing to do */
    }

    buf[0] = '\0';
    pwd = getpwuid(getuid());
    if (!pwd) {
	syslog(LOG_ERR, "getpwuid failed for uid %d: %m", getuid());
	pwd = getpwuid(geteuid());
    }
    if (!pwd) {
	syslog(LOG_ERR, "getpwuid failed for effective uid %d: %m", geteuid());
	sendClient("sOwner", "%s", tag);
    } else {
	if (pwd->pw_gecos) {
	    if (pwd->pw_gecos[0] == '&') {
		strcpy(buf, pwd->pw_name);
		strcat(buf, pwd->pw_gecos+1);
		if (islower(buf[0]))
		    buf[0] = toupper(buf[0]);
	    } else
		strcpy(buf, pwd->pw_gecos);
	    if ((cp = strchr(buf,',')) != 0)
		*cp = '\0';
	} else
	    strcpy(buf, pwd->pw_name);
    }
    if (debug) {
	if (*buf)
	     syslog(LOG_DEBUG, "fax user: \"%s\"", buf);
	else
	     syslog(LOG_DEBUG, "name of fax user unknown");
    }
    if (strcmp(requestor, buf) == 0) {
	(*f)(*job, tag, arg);
	if ((*job)->flags & JOB_INVALID)
	    *job = (*job)->next;
    } else
	sendClient("jobOwner", "%s", tag);
}
