#! /bin/sh
#	$Header: /usr/people/sam/fax/util/RCS/rts.sh,v 1.11 93/04/18 18:05:56 sam Rel $
#
# FlexFAX Facsimile Software
#
# Copyright (c) 1990, 1991, 1992, 1993 Sam Leffler
# Copyright (c) 1991, 1992, 1993 Silicon Graphics, Inc.
# 
# Permission to use, copy, modify, distribute, and sell this software and 
# its documentation for any purpose is hereby granted without fee, provided
# that (i) the above copyright notices and this permission notice appear in
# all copies of the software and related documentation, and (ii) the names of
# Sam Leffler and Silicon Graphics may not be used in any advertising or
# publicity relating to the software without the specific, prior written
# permission of Sam Leffler and Silicon Graphics.
# 
# THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
# EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
# WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
# 
# IN NO EVENT SHALL SAM LEFFLER OR SILICON GRAPHICS BE LIABLE FOR
# ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
# OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
# WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
# LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
# OF THIS SOFTWARE.
#

#
# rts [-t*] qfile server-pid number
#
# Return to sender information about a failed facsimile job.
# If the -t option is specified, a transcript of the call is
# included.  Otherwise the contents of the job's queue file
# are parsed and any unsent files are returned in a compressed
# archive file.
#
TAR=tar
AWK=nawk
ENCODE=uuencode
DECODE=uudecode
COMPRESS=compress
DECOMPRESS=zcat

PATH=/bin:/usr/bin:
test -d /usr/ucb  && PATH=$PATH:/usr/ucb		# Sun and others
test -d /usr/bsd  && PATH=$PATH:/usr/bsd		# Silicon Graphics
test -d /usr/5bin && PATH=/usr/5bin:$PATH:/usr/etc	# Sun and others
test -d /usr/sbin && PATH=/usr/sbin:$PATH		# 4.4BSD-derived

transcript=
case "$1" in
-t*) transcript=yes; shift;;
esac
if [ $# != 3 ]; then
    echo "Usage: rts [-t] qfile pid number"
    exit 1
fi
test "$transcript" && { ./bin/transcript $2 $3; }
qfile=$1

# look for an an awk: nawk, gawk, awk
($AWK '{}' </dev/null >/dev/null) 2>/dev/null ||
    { AWK=gawk; ($AWK '{}' </dev/null >/dev/null) 2>/dev/null || AWK=awk; }

$AWK -F: '
BEGIN		{ nfiles = 0; }
/^number/	{ number = $2; }
/^sender/	{ sender = $2; }
/^mailaddr/	{ mailaddr = $2; }
/^status/	{ status = $2; }
/^resolution/	{ resolution = $2;
		  if (resolution == 196)
		      resopt = " -m";
		}
/^npages/	{ npages = $2; }
/^dirnum/	{ dirnum = $2; }
/^ntries/	{ ntries = $2; }
/^ndials/	{ ndials = $2; }
/^pagewidth/	{ pagewidth = $2; }
/^pagelength/	{ pagelength = $2; }
/^modem/	{ modem = $2; }
/^notify/	{ if ($2 == "when done")
		      notify = " -D";
		  else if ($2 == "when requeued")
		      notify = " -R";
		}
/^[!]*post/	{ split($2, parts, "/");
		  if (!match(parts[2], "\.cover")) {	# skip cover pages
		     files = files " " parts[2];
		     nfiles++;
		  }
		}
/^!tiff/	{ split($2, parts, "/"); files = files " " parts[2]; nfiles++; }
/^poll/		{ poll = " -p"; }
END {
    printf "\n    ---- Unsent job status ----\n\n"
    printf "%11s: %s\n", "Destination", number;
    printf "%11s: %s\n", "Sender", sender;
    printf "%11s: %s\n", "Mailaddr", mailaddr;
    if (modem != "any")
	printf "%11s: %s\n", "Modem", modem;
    printf "%11s: %u (mm)\n", "PageWidth", pagewidth;
    printf "%11s: %.0f (mm)\n", "PageLength", pagelength;
    printf "%11s: %.0f (lpi)\n", "Resolution", resolution;
    printf "%11s: %s\n", "Status", status;
    printf "%11s: %u (consecutive failed calls to destination)\n",
	"Dials", ndials;
    printf "%11s: %u (attempts to send current page)\n",
	"Attempts", ntries;
    printf "%11s: %u (directory of next page to send)\n",
	"Dirnum", dirnum;
    if (nfiles > 0) {
	printf "\n    ---- Unsent files submitted for transmission ----\n\n";
	printf "This archive was created with %s, %s, and %s.\n",
	    tar, compressor, encoder;
	printf "The original files can be recovered by applying the following commands:\n";
	printf "\n";
	printf "    %s			# generates rts.%s.Z file\n", decoder, tar;
	printf "    %s rts.%s.Z  | %s xvf -	# generates separate files\n",
	    decompressor, tar, tar;
	printf "\n";
	printf "and the job can be resubmitted using the following command:\n";
	printf "\n";
	printf "    sendfax -d %s" poll notify resopt "%s\n", number, files;
	printf "\n";

	system("cd docq;" tar " cf - " files \
			" | " compressor \
			" | " encoder " rts." tar ".Z");
    }
}
' tar=$TAR encoder=$ENCODE decoder=$DECODE \
  compressor=$COMPRESS decompressor=$DECOMPRESS \
  $qfile 2>/dev/null || {
      echo ""
      echo "Sorry, unable to return unsent files -- problem in $0."
      echo ""
  }
